# Generated by roxygen2: do not edit by hand

S3method(numeral_formatter,"function")
S3method(numeral_formatter,character)
S3method(numeral_formatter,default)
S3method(numeral_formatter,numeric)
S3method(print,statlist)
S3method(print,tidytlg.freq)
S3method(print,tidytlg.nested_freq)
S3method(print,tidytlg.univar)
export(add_bottom_borders)
export(add_format)
export(add_indent)
export(add_newrows)
export(bind_table)
export(char2factor)
export(col_borders)
export(freq)
export(generate_results)
export(gentlg)
export(get_file_name)
export(insert_empty_rows)
export(nested_freq)
export(no_borders)
export(replace_na_with_blank)
export(rmdpstitle)
export(roundSAS)
export(row_border)
export(single_border)
export(spanning_borders)
export(spanning_headers)
export(statlist)
export(tidytlg_metadata)
export(tidytlg_titles)
export(tlgsetup)
export(univar)
importFrom(cellranger,cell_cols)
importFrom(crayon,bold)
importFrom(crayon,underline)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,ends_with)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_at)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(forcats,fct_expand)
importFrom(forcats,fct_recode)
importFrom(forcats,fct_relevel)
importFrom(ggplot2,ggsave)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(huxtable,everywhere)
importFrom(magrittr,"%>%")
importFrom(magrittr,extract2)
importFrom(methods,formalArgs)
importFrom(png,readPNG)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(purrr,map_lgl)
importFrom(purrr,pmap)
importFrom(purrr,walk)
importFrom(readxl,read_excel)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,as_name)
importFrom(rlang,as_string)
importFrom(rlang,enexpr)
importFrom(rlang,enexprs)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,exprs)
importFrom(rlang,is_call)
importFrom(rlang,is_named)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rstudioapi,getSourceEditorContext)
importFrom(rstudioapi,isAvailable)
importFrom(stats,lag)
importFrom(stats,setNames)
importFrom(stringr,regex)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace)
importFrom(stringr,str_split)
importFrom(stringr,str_to_lower)
importFrom(tibble,add_row)
importFrom(tibble,tibble)
importFrom(tibble,tibble_row)
importFrom(tibble,tribble)
importFrom(tidyr,complete)
importFrom(tidyr,crossing)
importFrom(tidyr,fill)
importFrom(tidyr,nest)
importFrom(tidyr,nesting)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,unnest)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,tail)
